static const char file_id[] = "SRItclStar.cc";

/*  Version @(#)SRItclStar.cc	1.14 06/05/97

@Copyright (c) 1996-1997 The Regents of the University of California.
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in all
copies of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

						PT_COPYRIGHT_VERSION_2
						COPYRIGHTENDKEY

    Author:     S. A. Edwards
    Created:    14 April 1996

*/

#ifdef __GNUG__
#pragma implementation
#endif

#include "SRItclStar.h"
#include "InfString.h"
#include "StringMessage.h"
#include "HashTable.h"
#include "PortHole.h"
#include "Star.h"
#include <stream.h>

// The main interpreter used by ptcl
// @Description  This should, perhaps, not be used as the interpreter,
// but is the easiest way to do it for the moment
extern Tcl_Interp * ptkInterp;

HashTable SRItclStar::starTable;

// No interpreter connected with the system yet
Tcl_Interp * SRItclStar::interp = 0;

// Return a string describing the contents of the port
//
// @Description If the value is present, return a printed version of
// the particle;
// if the value is absent, return "absent"; and
// if the value is unknown, return "unknown".

int SRItclStar::readPort( PortHole * p )
{
  SRPortHole * port = (SRPortHole *) p;

  if ( port->present() ) {
    InfString theValue = (port->get()).print();
    Tcl_SetResult( interp, (char *) theValue, TCL_VOLATILE );
  } else if ( port->absent() ) {
    interp->result = "absent";
  } else {
    interp->result = "unknown";
  }

  return TCL_OK;
}

// Write a string to an OutSRPort
//
// @Description If writing "absent," mark the output as absent;
// otherwise, write out the value.

int SRItclStar::writePort( PortHole * p, char * data )
{
  OutSRPort * port = (OutSRPort *) p;

  if ( strcmp(data, "unknwown") != 0 ) {
    if ( strcmp(data, "absent") == 0 ) {
      port->makeAbsent();
    } else {
      // FIXME: This only works for "string" type outputs
      port->emit() << Envelope( *(new StringMessage( data )) );
    }
  }

  return TCL_OK;

}

// Construct the object associated with this star
//
// @Description The name of the class comes from the itclClassName state.
// the name of the object is either itclObjectName (if it's non-empty),
// or the value of the itclObjectName state.  Expand the pathname of the
// source file using expandPathName.

void SRItclStar::begin()
{
  char * filename = expandPathName( (const char *) itclSourceFile );

  int result;

  if ( (const char *) itclObjectName == NULL ||
       strcmp( (const char *) itclObjectName, "" ) == 0 ) {
    result = construct( (const char *) itclClassName, name(), filename );
  } else {
    result = construct( (const char *) itclClassName,
			(const char *) itclObjectName,
			filename );
  }

  delete [] filename;

  if ( result != TCL_OK ) {
    Error::abortRun( *this, interp->result );
  }
}

// Destroy the itcl object associated with this star by calling destroy()
void SRItclStar::wrapup()
{
  destroy();
}

//////////////////////////////////////////////////////////////////////
//
// Callback commands for itcl methods
//
// To add a command, add the method name and function pointer to the
// ItclMethodList array, defined after the definitions

/*
 *----------------------------------------------------------------------
 * SRItclStar_ReadCmd
 * 
 * Returns the value on a port.  Usage:
 *
 * <starObject> read <portName>
 *
 *----------------------------------------------------------------------
 */
/* SRItclStar_ReadCmd was static, but compiling with -O3 results
 * in missing symbol errors.
 */
int
SRItclStar_ReadCmd(ClientData		/* unused */,
		   Tcl_Interp * interp	/* interpreter */,
		   int argc		/* number of arguments */,
		   char ** argv		/* array of arguments */
		   )
{

  if ( argc != 2 ) {
    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
		     "\" portName ", (char *) NULL );
    return TCL_ERROR;
  }

  // Find the star whose input we are reading

  SRItclStar * theStar = SRItclStar::starOfCurrentObject( interp );
  if ( theStar == 0 ) {
    return TCL_ERROR;
  }

  // Find the port on that star

  PortHole * thePort = theStar->portWithName( argv[1] );
  if ( thePort == 0 ) {
    Tcl_AppendResult( interp, "No port named ", argv[1], (char *) NULL );
    return TCL_ERROR; 
  }

  /*
  if ( !(thePort->isItInput()) ) {
    Tcl_AppendResult( interp, "Port ", argv[1], " is not an input",
		      (char *) NULL );
    return TCL_ERROR;
  }
  */

  return theStar->readPort( thePort );
}

/*
 *----------------------------------------------------------------------
 * SRItclStar_WriteCmd
 * 
 * Writes a value to a port.  Usage:
 *
 * <starObject> write <portName> <newValue>
 *
 *----------------------------------------------------------------------
 */
int
SRItclStar_WriteCmd(ClientData		/* unused */,
		    Tcl_Interp * interp	/* interpreter */,
		    int argc		/* number of arguments */,
		    char ** argv		/* array of arguments */
		    )
{

  if ( argc != 3 ) {
    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
		     "\" portName outputValue", (char *) NULL );
    return TCL_ERROR;
  }

  // Find the star whose output we are writing

  SRItclStar * theStar = SRItclStar::starOfCurrentObject( interp );
  if ( theStar == 0 ) {
    return TCL_ERROR;
  }

  // Find the port on that star

  PortHole * thePort = theStar->portWithName( argv[1] );
  if ( thePort == 0 ) {
    Tcl_AppendResult( interp, "No port named ", argv[1], (char *) NULL );
    return TCL_ERROR; 
  }

  if ( !(thePort->isItOutput()) ) {
    Tcl_AppendResult( interp, "Port ", argv[1], " is not an output",
		      (char *) NULL );
    return TCL_ERROR;
  }

  return theStar->writePort( thePort, argv[2] );
}

/*
 *----------------------------------------------------------------------
 * SRItclStar_StarNameCmd
 * 
 * Returns the name of the star in the galaxy.  Usage:
 *
 * <starObject> starName
 *
 *----------------------------------------------------------------------
 */
/* SRItclStar_StarNameCmd was static, but compiling with -O3 results
 * in missing symbol errors.
 */
int
SRItclStar_StarNameCmd(ClientData		/* unused */,
		       Tcl_Interp * interp	/* interpreter */,
		       int argc			/* number of arguments */,
		       char ** argv		/* array of arguments */
		       )
{

  if ( argc != 1 ) {
    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
		     "\"", (char *) NULL );
    return TCL_ERROR;
  }

  // Find the star whose name we are returning

  SRItclStar * theStar = SRItclStar::starOfCurrentObject( interp );
  if ( theStar == 0 ) {
    return TCL_ERROR;
  }

  char * theName = savestring( theStar->name());

  Tcl_SetResult( interp, theName, TCL_VOLATILE );
  delete [] theName;
  
  return TCL_OK;
}

//////////////////////////////////////////////////////////////////////
//
// Array of methods to be registered
//

typedef struct ItclMethod {
  char * name;		// name of the method to be registered
  Tcl_CmdProc * proc;	// C routine implementing the method
} itclMethod;

static const itclMethod ItclMethodList[] = {
  { "SRItclStar-read",		SRItclStar_ReadCmd },
  { "SRItclStar-write",		SRItclStar_WriteCmd },
  { "SRItclStar-starName",	SRItclStar_StarNameCmd }
};

// Constructor
//
// @Description Build an SRItclStar that is unattached to any itcl
// object.  Binding done with construct().

SRItclStar::SRItclStar()
{

  // Initialize the SRItclSystem if it hasn't already been initialized
  initialize( ptkInterp );

  // No associated Itcl object yet
  objectName = 0;
  itclObject = 0;

  addState( itclClassName.setState("itclClassName", this, "SRItclStar",
				   "The itcl class of this star") );

  addState( itclSourceFile.setState("itclSourceFile", this,
				    "$PTOLEMY/src/domains/sr/stars/SRItclStar.itcl",
				    "The pathname of the itcl source file "
				    "containing the class for this star."));

  addState( itclObjectName.setState("itclObjectName", this, "",
				    "The name of the itcl object to create. "
				    "Defaults to the instance name.") );


  // cout << "Constructed an SRItclStar\n";
}

// Destructor
//
// @Description Destroy the associated itcl object

SRItclStar::~SRItclStar()
{
  destroy();
  // cout << "Destructed an SRItclStar\n";
}

// Return the Star associated with the current itcl object
//
// @Description Find the star associated with the current object by looking up
// the "this" variable in the starTable hash table.  If the star
// doesn't exist, return 0 and set the error result to explain the problem

SRItclStar * SRItclStar::starOfCurrentObject( Tcl_Interp * interp )
{
  char * objectName;
  SRItclStar * theStar;

  if ( (objectName = Tcl_GetVar( interp, "this", 0 ) ) != 0 ) {
    theStar = (SRItclStar *) starTable.lookup( objectName );
    if ( theStar == 0 ) {
      Tcl_AppendResult( interp, "Could not look up interface to ", objectName,
			(char *) NULL );
    } else {
      if ( theStar->interp != interp ) {
	Tcl_AppendResult( interp, "Interface to ", objectName,
			  " resides in different interpreter", (char *) NULL );
	return 0;
      }
    }
    return theStar;
  }

  Tcl_AppendResult( interp, "Could not evaluate $this", (char *) NULL );
  return 0;
}

// Initialize the SRItclStar system
//
// @Description Register commands with the Tcl interpreter

int SRItclStar::initialize( Tcl_Interp * initializingInterp )
{

  if ( interp ) {
    return TCL_OK;
  }

  if ( initializingInterp == 0 ) {
    // cout << "Tried to initialize the SRItclStar system with a vacuous interpreter\n";
    return TCL_OK;
  }

  // cout << "Initializing the SRItclStar system with interp "
  //     << initializingInterp << '\n';

  interp = initializingInterp;

  // Register the method commands

  unsigned int i;
  for ( i = 0 ; i < sizeof(ItclMethodList) / sizeof(ItclMethod); i++ ) {
    if ( Itcl_RegisterC( interp, ItclMethodList[i].name,
			 ItclMethodList[i].proc ) != TCL_OK ) {
      return TCL_ERROR;
    }
  }

  return TCL_OK;    
}


// Construct a Itcl object and bind it to the star
//
// @Description First, try to destroy any existing Itcl object, then
// source the file containing the class if the given pointer is non-NULL,
// and create the itcl object in the global namespace.

int SRItclStar::construct( const char * className,
			   const char * newObjectName,
			   const char * sourceFile
			   /* Pathname of source file containing the class.  May be NULL */,
			   int argc , char *argv[] )
{
  // Try to destroy any attached Itcl object first.

  destroy();

  // cout << "Calling construct on " << className << " " << newObjectName << "\n";

  InfString buf;

  // Delete the class if it exists, then source the script containing
  // the class definition

  if ( itclSourceFile != NULL && strcmp(itclSourceFile,"") != 0 ) {

    // Delete the class.  If this returns an error, ignore it.

    buf = "delete class ";
    buf += className;
    Tcl_GlobalEval( interp, (char *) buf );

    // Source the filename

    buf = "source ";
    buf += sourceFile;
    if ( Tcl_GlobalEval( interp, (char *) buf ) != TCL_OK ) {
      return TCL_ERROR;
    }
  }     

  // Build up a Tcl command to create the object:
  // className newObjectName argv[0] argv[1] .. argv[argc-1]

  buf = className;
  buf += " ";
  buf += newObjectName;
  for ( int i = 0 ; i < argc ; i++ ) {
    buf += " ";
    buf += argv[i];
  }

  // Create the object
  //
  // FIXME: In what namespace should these things be created?

  if ( Tcl_GlobalEval( interp, (char *) buf ) != TCL_OK ) {
    return TCL_ERROR;
  }

  // Find the object we just created

  if ( Itcl_FindObject( interp, interp->result, &itclObject ) != TCL_OK ) {
    return TCL_ERROR;
  }

  // Get its full path name

  if ( itclObject->accessCmd ) {
    objectName = Tcl_GetCommandPath( interp, itclObject->accessCmd );
  } else {
    interp->result = "Unable to get pathname";
    return TCL_ERROR;
  }

  // Save the name of this object and this interface in the hash table

  objectName = savestring( objectName );
  starTable.insert( objectName, (Pointer) this );

  return TCL_OK;
}

// Destroy the Itcl object associated with this star
//
// @Description Have Itcl destroy it, remove it from the hash table

int SRItclStar::destroy()
{

  if ( objectName ) {

    // cout << "Calling destroy on " << objectName << "\n";

    InfString buf;

    // Build up a Tcl command to delete the itcl object
    
    buf = "delete object ";
    buf += objectName;

    // Delete the itcl object
    //
    // FIXME: In what namespace should this be done?

    if ( Tcl_GlobalEval( interp, (char *) buf ) != TCL_OK ) {
      return TCL_ERROR;
    }

    itclObject = 0;

    // Remove the name from the hash table

    starTable.remove(objectName);
    delete [] objectName;
    objectName = 0;

  }

  return TCL_OK;

}

// Call the itcl object's go method with no arguments
void SRItclStar::go()
{
  if ( objectName ) {
    InfString buf;
    buf = objectName;
    buf += " go";
    Tcl_GlobalEval( interp, (char *) buf );
  }

}

// Call the itcl object's tick method with no arguments
void SRItclStar::tick()
{
  if ( objectName ) {
    InfString buf;
    buf = objectName;
    buf += " tick";
    Tcl_GlobalEval( interp, (char *) buf );
  }

}

